import activityRouter from "./activity/router";
import adminRouter from "./admin/router";
import authRouter from "./auth/router";
import clientsRouter from "./clients/router";
import configRouter from "./config/router";
import gamesRouter from "./games/router";
import importRouter from "./import/router";
import importsRouter from "./imports/router";
import oauthRouter from "./oauth/router";
import scoresRouter from "./scores/router";
import searchRouter from "./search/router";
import seedsRouter from "./seeds/router";
import sessionsRouter from "./sessions/router";
import statusRouter from "./status/router";
import usersRouter from "./users/router";
import { Router } from "express";
import { NormalRateLimitMiddleware } from "server/middleware/rate-limiter";

const router: Router = Router({ mergeParams: true });

// Auth is up here so it can have special rate limiting rules,
// since it needs slightly harsher ones!
router.use("/auth", authRouter);

// Everything else can use the normal rate limiter!
router.use(NormalRateLimitMiddleware);

router.use("/admin", adminRouter);
router.use("/activity", activityRouter);
router.use("/status", statusRouter);
router.use("/import", importRouter);
router.use("/imports", importsRouter);
router.use("/users", usersRouter);
router.use("/games", gamesRouter);
router.use("/search", searchRouter);
router.use("/scores", scoresRouter);
router.use("/sessions", sessionsRouter);
router.use("/oauth", oauthRouter);
router.use("/clients", clientsRouter);
router.use("/config", configRouter);
router.use("/seeds", seedsRouter);

/**
 * Return a JSON 404 response if an endpoint is hit that does not exist.
 *
 * @name ALL /api/v1/*
 */
router.all("*", (req, res) =>
	res.status(404).json({
		success: false,
		description: "Endpoint Not Found.",
	})
);

export default router;
